using System;
using System.Text;
using System.Collections;
using System.Drawing;
using System.IO;
using System.Runtime.InteropServices;
using System.Reflection;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text.RegularExpressions;
using System.Xml;


namespace PluginLib
{
	#region CMapperXmlTransformer
	public class CMapperXmlTransformer
	{
		public CMapperXmlTransformer ( )
		{
		}

		#region Tags
		private static string m_tagset = "property_set";
		private static string m_tagcoll = "property_collection";
		private static string m_tagdata = "data";
		private static string m_tagpropsimple = "property_simple";
		private static string m_tagpropcdata = "property_cdata";
		private static string m_tagstart = "oktellxmlmapper";
		#endregion

		#region Encoding
		/// <summary>
		///   
		/// </summary>
		public enum EMapperXmlEncoding
		{
			Default					= 0,
			ASCII					= 1,
			BigEndianUnicode		= 2,
			UTF7					= 3,
			UTF8					= 4,
			UTF16					= 5
		}

		/// <summary>
		///   
		/// </summary>
		private static Encoding m_CurrentEncoding = Encoding.Unicode;

		/// <summary>
		///      
		/// </summary>
		/// <returns></returns>
		public static Encoding SetCurrentEncoding ( EMapperXmlEncoding type )
		{
			switch ( type )
			{
				case EMapperXmlEncoding.Default:			return Encoding.Default;
				case EMapperXmlEncoding.ASCII:				return Encoding.ASCII;
				case EMapperXmlEncoding.BigEndianUnicode:	return Encoding.BigEndianUnicode;
				case EMapperXmlEncoding.UTF7:				return Encoding.UTF7;
				case EMapperXmlEncoding.UTF8:				return Encoding.UTF8;
				case EMapperXmlEncoding.UTF16:				return Encoding.Unicode;
				default:									return Encoding.Unicode;
			}
		}

		/// <summary>
		///     
		/// </summary>
		private static Encoding GetCurrentEncoding ( )
		{
			return m_CurrentEncoding;			
		}
		#endregion

		#region Write

		/// <summary>
		///    XML        
		/// </summary>
		/// <returns></returns>
		private static double GetVersion ( )
		{
			return 080710;
		}

		#region Inner Xml
		/// <summary>
		///  MemoryStream,          DATA
		/// </summary>
		/// <returns>  XML</returns>
		private static XmlTextWriter _CreateXmlHeader ( MemoryStream ms )
		{	
			XmlTextWriter writer = new XmlTextWriter ( ms, GetCurrentEncoding() );
			writer.Formatting = Formatting.Indented;			
			
			//document
			writer.WriteStartDocument ( );
			writer.WriteStartElement ( m_tagstart );
			writer.WriteAttributeString ( "version", GetVersion().ToString ( ) );
			
			//data
			writer.WriteStartElement ( m_tagdata );

			return writer;
		}

		/// <summary>
		///  MemoryStream,          DATA
		/// </summary>
		/// <returns>  XML</returns>
		private static XmlTextWriter _CreateXmlHeader ( string path )
		{	
			XmlTextWriter writer = new XmlTextWriter ( path, GetCurrentEncoding() );
			writer.Formatting = Formatting.Indented;			
			
			//document
			writer.WriteStartDocument ( );
			writer.WriteStartElement ( m_tagstart );
			writer.WriteAttributeString ( "version", GetVersion().ToString ( ) );
			
			//data
			writer.WriteStartElement ( m_tagdata );

			return writer;
		}
		

		/// <summary>
		///   XML   /DATA   
		/// </summary>
		private static void _FinishXml ( XmlTextWriter writer )
		{
			//Data
			writer.WriteEndElement ( );

			//oktellcommapper
			writer.WriteEndElement ( );
			//document
			writer.WriteEndDocument ( );
			
			writer.Close ( );
		}
		/// <summary>
		///   xml   .
		///       
		/// </summary>
		private static void _InsertXmlSet ( XmlTextWriter writer, CMapperXmlItemSet items )
		{			
			if ( items != null )
			{
				//     XML  
				#region   
				foreach ( CMapperXmlItem e in items )
				{				
					//  ( key="newstate")
					string key = e.Key;
					//  ( value="5")
					string val = e.Value;
					//  ( name="fullbusy")
					string text = e.Name;				
				
					switch ( e.Type )
					{
							#region Simple							
						case CMapperXmlItem.EItemType.Simple:
						{
							//<property_simple key="" value="" name=""/>
							writer.WriteStartElement ( m_tagpropsimple );
				
							if ( key != null )
								writer.WriteAttributeString ( "key", key );
							if ( val != null )
								writer.WriteAttributeString ( "value", val );
							if ( text != null )
								writer.WriteAttributeString ( "name", text );
				
							writer.WriteEndElement ( );

							break;
						}
							#endregion

							#region CData							
						case CMapperXmlItem.EItemType.CData:
						{
							//<property_cdata key="" value="">[![CDATA[""]]</property_cdata>
							writer.WriteStartElement ( m_tagpropcdata );
				
							if ( key != null )
								writer.WriteAttributeString ( "key", key );
							if ( val != null )
								writer.WriteAttributeString ( "value", val );					
							if ( text != null )
								writer.WriteCData ( text );
				
							writer.WriteEndElement ( );

							break;
						}
							#endregion
					}
				}
				#endregion

				#region 
				if ( items.SubSets.Count > 0 )
				{
					// 
					writer.WriteStartElement ( m_tagcoll );
					string name = items.SubSets.Name;
					if (( name != null ) && ( name != String.Empty ))
						writer.WriteAttributeString ( "name", name );
					writer.WriteAttributeString ( "count", items.SubSets.Count.ToString() );

					// 
					_InsertXmlColl ( writer, items.SubSets );

					// 
					writer.WriteEndElement ( );
				}
				#endregion
			}			
		}

		/// <summary>
		///   xml   .
		///      
		/// </summary>
		private static void _InsertXmlColl ( XmlTextWriter writer, CMapperXmlItemSetCollection items )
		{			
			if ( items != null )
			{
				//     XML  
				foreach ( CMapperXmlItemSet e in items )
				{
					// 
					writer.WriteStartElement ( m_tagset );
					if (( e.Name != null ) && ( e.Name != String.Empty ))
						writer.WriteAttributeString ( "name", e.Name );
					if (( e.IdName != null ) && ( e.IdName != String.Empty ))
						writer.WriteAttributeString ( "id", e.IdName );

					// 
					_InsertXmlSet ( writer, e );
					
					// 
					writer.WriteEndElement ( );
				}
			}			
		}

		#endregion 

		#region GetXmlString (+ overloads)
		
		/// <summary>
		///  XML ,        - 
		/// </summary>
		/// <param name="eventtype"> </param>
		/// <param name="items">  CMapperXmlItem</param>		
		/// <returns>  UTF-16,  XML- </returns>
		public static string GetXmlString ( string description, CMapperXmlItemSet items )
		{
			MemoryStream ms = new MemoryStream ( );
			
			//
			XmlTextWriter writer = _CreateXmlHeader ( ms );
			
			//   <data>
			writer.WriteAttributeString ( "name", description );
						
			// 
			_InsertXmlSet ( writer, items );
			
			//
			_FinishXml ( writer );

			string xmlstring = GetCurrentEncoding().GetString ( ms.ToArray () );
			
			//   -  ..
			//			if ( m_XmlLogging )
			//				BaseStatic.CommonLog ( "XML event parameter", "\n" + xmlstring + "\n" );

			return xmlstring;
		}

		/// <summary>
		///   XML        
		/// </summary>
		/// <param name="description">,     name</param>
		/// <param name="items"> </param>
		/// <returns>  UTF-16,  XML- </returns>
		public static string GetXmlString ( string description, CMapperXmlItemSetCollection items )
		{
			MemoryStream ms = new MemoryStream ( );
			
			//
			XmlTextWriter writer = _CreateXmlHeader ( ms );
			
			int itemscnt = 0;
			if ( items != null )
				itemscnt = items.Count;
			
			//   <data>
			writer.WriteAttributeString ( "name", description );
			writer.WriteAttributeString ( "count", itemscnt.ToString() );
			
			// 
			_InsertXmlColl ( writer, items );
						
			//
			_FinishXml ( writer );

			
			string xmlstring = GetCurrentEncoding().GetString ( ms.ToArray () );
			
			//   -  ..
			//			if ( m_XmlLogging )
			//				BaseStatic.CommonLog ( "XML out parameter", "\n" + xmlstring + "\n" );

			return xmlstring;
		}		

		/// <summary>
		///   XML (  )        
		/// </summary>
		/// <param name="description">,     name</param>
		/// <param name="items"> </param>
		/// <param name="items">  </param>
		public static void CreateXmlFile ( string description, CMapperXmlItemSetCollection items, string path )
		{
			// -
			if ( File.Exists ( path ) )
				File.Delete ( path );
			
			//
			XmlTextWriter writer = _CreateXmlHeader ( path );
						
			//   <data>
			writer.WriteAttributeString ( "name", description );
			writer.WriteAttributeString ( "count", items.Count.ToString() );
			
			// 
			_InsertXmlColl ( writer, items );
						
			//
			_FinishXml ( writer );			
		}
		#endregion

		#endregion


		#region Parse

		#region Inner
		private static void _ParseDataChapter ( XmlTextReader reader, CMapperXmlItemSetCollection basecoll )
		{
			CMapperXmlItemSetCollection currentColl = basecoll;
			CMapperXmlItemSet currentSet = null;
			
			// ,     ( DATA = SET )
			CMapperXmlItemSet errorset = new CMapperXmlItemSet ( );
			errorset.ParentCollection = basecoll;			
			errorset.Name = reader.GetAttribute ( "name" );
			basecoll.Add ( errorset );
			currentSet = errorset;

			while ( ! reader.EOF )
			{
				reader.Read ( );
				while (( ! reader.EOF ) && 
					( reader.NodeType != XmlNodeType.Element ) &&
					( reader.NodeType != XmlNodeType.EndElement ))
					reader.Read ( );

				if ( reader.EOF )
					break;
				
				switch ( reader.NodeType )
				{
					case XmlNodeType.Element:
						#region Element
						if ( reader.Name == m_tagcoll )
						{
							string name = reader.GetAttribute ( "name" );
							string count = reader.GetAttribute ( "count" );

							//      , 
							//      data   
							CMapperXmlItemSetCollection setcoll = null;
							if (( currentColl == basecoll ) && ( currentSet == null ))
								setcoll = basecoll;
							else							
								setcoll = currentSet.SubSets;							
					
							setcoll.Name = name;
							currentColl = setcoll;
						}
						else if ( reader.Name == m_tagset )
						{
							// , DATA=COLL,    ERRORSET
							if (( currentSet == errorset ) && ( currentColl == basecoll ))
								basecoll.Remove ( errorset );

							string name = reader.GetAttribute ( "name" );
							string idname = reader.GetAttribute ( "id" );
							
							CMapperXmlItemSet itemset = new CMapperXmlItemSet ( );
							itemset.Name = name;
							itemset.IdName = idname;
							itemset.ParentCollection = currentColl;

							currentSet = itemset;
							currentColl.Add ( itemset );
						}
						else if ( reader.Name == m_tagpropsimple )
						{
							string key = reader.GetAttribute ( "key" );
							string val = reader.GetAttribute ( "value" );
							string name = reader.GetAttribute ( "name" );

							CMapperXmlItem item = new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key, val, name );

							currentSet.Add ( item );
						}
						else if ( reader.Name == m_tagpropcdata )
						{
							string key = reader.GetAttribute ( "key" );
							string val = reader.GetAttribute ( "value" );
							string text = null;
							if ( ! reader.IsEmptyElement )
							{
								XmlNodeType type = reader.MoveToContent ( );
								if ( type == XmlNodeType.CDATA )
									text = reader.Value;
								else if ( type == XmlNodeType.Element )
								{
									while (( reader.NodeType != XmlNodeType.CDATA ) &&
										( reader.NodeType != XmlNodeType.EndElement ))
										reader.Read();
							
									if ( reader.NodeType == XmlNodeType.CDATA )
										text = reader.Value;
								}
							}

							CMapperXmlItem item = new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key, val, text );

							currentSet.Add ( item );
						}
						break;
						#endregion

					case XmlNodeType.EndElement:
						#region EndElement
						if ( reader.Name == m_tagcoll )
						{
							currentColl = currentSet.ParentCollection;
						}
						else if ( reader.Name == m_tagset )
						{
							currentSet = currentColl.ParentSet;
						}
						break;
						#endregion
				}
			}
		}
		
		#endregion

		#region ParseXmlString

		private static CMapperXmlItemSetCollection _ParseXmlReader ( XmlTextReader reader )
		{
			if ( reader == null )
				return null;

			// 
			CMapperXmlItemSetCollection coll = new CMapperXmlItemSetCollection ( );
			double version = GetVersion ( );

			//        (      )
			while ( ! reader.EOF )
			{
				reader.Read ( );
				while (( ! reader.EOF ) && ( reader.NodeType != XmlNodeType.Element ))
					reader.Read ( );

				if ( reader.EOF )
					break;

				if ( reader.Name.ToLower() == m_tagstart )
					version = double.Parse ( reader.GetAttribute ( "version" ) );
				
				else if ( reader.Name.ToLower() == m_tagdata )
				{					
					string name = reader.GetAttribute ( "name" );
					coll.Name = name;

					_ParseDataChapter ( reader, coll );						
				}						
			}

			return coll;
		}

		public static CMapperXmlItemSetCollection ParseXmlString ( string xml )
		{
			//  
			if ( xml == null )			
				return null;			
			
			XmlTextReader reader = null;

			try
			{
				//    XmlTextReader
				MemoryStream ms = new MemoryStream ( GetCurrentEncoding().GetBytes ( xml ) );
				reader = new XmlTextReader ( ms );
				reader.WhitespaceHandling = WhitespaceHandling.None;

				return _ParseXmlReader ( reader );
			}
			catch ( Exception ex )
			{
				//Log.Write ( ex.ToString() );
			}
			finally
			{
				if ( reader != null )
					reader.Close ( );
			}
			
			return null;
		}

		public static CMapperXmlItemSetCollection ParseXmlFile ( string path )
		{
			//  
			if ( path == null )			
				return null;
			
			if ( ! File.Exists ( path ) )
				return null;

			XmlTextReader reader = null;

			try
			{
				reader = new XmlTextReader ( path );
				reader.WhitespaceHandling = WhitespaceHandling.None;

				return _ParseXmlReader ( reader );
			}
			catch ( Exception ex )
			{
				//Log.Write ( ex.ToString() );
			}
			finally
			{
				if ( reader != null )
					reader.Close ( );
			}

			return null;
		}
		#endregion

		#endregion
	}
	#endregion

	#region CMapperXmlItem
	[Serializable]
	public class CMapperXmlItem
	{
		public enum EItemType
		{
			Simple,		//<property_simple key="" value="" name=""/>
			CData		//<property_cdata key="" value="">[![CDATA[""]]</property_cdata>
		}

		private EItemType m_Type = EItemType.Simple;
		public EItemType Type
		{
			get { return m_Type; }
		}

		public string Key = null;
		public string Value = null;
		public string Name = null;

		public CMapperXmlItem ( EItemType type, string key, string val, string name )
		{
			m_Type = type;

			Key = key;
			Value = val;
			Name = name;
		}
		
		public CMapperXmlItem ( EItemType type )
		{
			m_Type = type;
		}

		private static Regex regex;

		public static CMapperXmlItem GetItem ( string key, object val )
		{
			if ( regex == null )
				regex = new Regex ( @"[^A-Za-z0-9]+" );

			if ( val != null )
			{
				if ( val is Guid )
					return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), val.ToString(), null );

				if ( val is Boolean )
					return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), ((bool)val) ? "1" : "0", null );
				
				else if ( val is Enum )
					return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), Convert.ToInt32 ( val ).ToString(), val.ToString() );
				
				else if ( val is DateTime )
				{
					DateTime dt = (DateTime)val;

					if ( dt.Year >= 1980 )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), dt.ToString ( "dd.MM.yyyy HH:mm:ss" ), null );
					else
					{						
						TimeSpan sp = new TimeSpan ( dt.Ticks );
						string h = ((int) Math.Floor ( sp.TotalHours )).ToString("00");
						string m = sp.Minutes.ToString("00");
						string s = sp.Seconds.ToString("00");
						
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), h + ":" + m + ":" + s, null );
					}
				}
				
				else if ( val is TimeSpan )
					return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), ((TimeSpan)val).TotalMilliseconds.ToString(), null );
				
				else if ( val is string )
				{
					string sval = val as String;
					if (( regex.IsMatch ( sval ) ) || ( sval.Length > 50 ))
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), null, sval );
					else
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), sval, null );
				}
				
				else
					return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), val.ToString(), null );
			}

			return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), String.Empty, null );
		}

		public static CMapperXmlItem GetItem ( string key, object val, string name )
		{
			if ( regex == null )
				regex = new Regex ( @"[^A-Za-z0-9]+" );

			if ( val != null )
			{
				if (( regex.IsMatch ( name ) ) || ( name.Length > 50 ))
				{
					#region Val , Name - cdata
					
					if ( val is Guid )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), val.ToString(), name );
				
					else if ( val is Enum )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), Convert.ToInt32 ( val ).ToString(), name );
				
					else if ( val is DateTime )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), ((DateTime)val).ToString ( "dd.MM.yyyy HH:mm:ss" ), name );
				
					else if ( val is TimeSpan )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), ((TimeSpan)val).TotalMilliseconds.ToString(), name );
				
					else if ( val is string )
					{
						string sval = val as String;
						if (( regex.IsMatch ( sval ) ) || ( sval.Length > 50 ))
							return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), null, sval );
						else
							return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), sval, name );
					}
				
					else
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), val.ToString(), name );
					
					#endregion
				}
				else
				{
					#region Val , name 

					if ( val is Guid )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), val.ToString(), name );
				
					else if ( val is Enum )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), Convert.ToInt32 ( val ).ToString(), name );
				
					else if ( val is DateTime )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), ((DateTime)val).ToString ( "dd.MM.yyyy HH:mm:ss" ), name );
				
					else if ( val is TimeSpan )
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), ((TimeSpan)val).TotalMilliseconds.ToString(), name );
				
					else if ( val is string )
					{
						string sval = val as String;
						if (( regex.IsMatch ( sval ) ) || ( sval.Length > 50 ))
							return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), null, sval );
						else
							return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), sval, name );
					}
				
					else
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), val.ToString(), name );

					#endregion
				}
			}

			else
			{
				#region Val  

				if ( name != null )
				{
					if (( regex.IsMatch ( name ) ) || ( name.Length > 50 ))				
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.CData, key.ToLower(), null, name );
				
					else				
						return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), null, name );
				}
				else
				{
					return new CMapperXmlItem ( CMapperXmlItem.EItemType.Simple, key.ToLower(), null, "" );
				}

				#endregion
			}
		}


		public static string GetValueName ( CMapperXmlItem item )
		{
			if ( item.Value != null )
				return item.Value;
			else
				return item.Name;
		}

		public static string GetNameValue ( CMapperXmlItem item )
		{
			if ( item.Name != null )
				return item.Name;
			else
				return item.Value;
		}
	}	
	#endregion

	#region CMapperXmlItemSet
	[Serializable]
	public class CMapperXmlItemSet : IList
	{
		#region Fields & Properties

		private ArrayList list;
		
		private string m_Name = "";
		/// <summary>
		///     
		///    name  
		/// </summary>
		public string Name
		{
			get { return m_Name; }
			set { m_Name = value; }
		}
		
		private string m_IdName = "";
		/// <summary>
		/// Id    
		///    id  
		/// </summary>
		public string IdName
		{
			get { return m_IdName; }
			set { m_IdName = value; }
		}

		private CMapperXmlItemSetCollection m_ParentCollection = null;
		/// <summary>
		/// -   (ItemSetCollection).
		/// </summary>
		public CMapperXmlItemSetCollection ParentCollection
		{
			get { return m_ParentCollection; }
			set { m_ParentCollection = value; }
		}
		
		private CMapperXmlItemSetCollection m_SubSets = null;
		/// <summary>
		/// 
		/// </summary>
		public CMapperXmlItemSetCollection SubSets
		{
			get { return m_SubSets; }
			set { m_SubSets = value; }
		}

		#endregion

		#region Constructor

		public CMapperXmlItemSet ( ):base ( )
		{
			list = new ArrayList ( );

			m_SubSets = new CMapperXmlItemSetCollection ( );
			m_SubSets.ParentSet = this;
		}
		#endregion

		#region Xml
		public static string ToXml ( string description, CMapperXmlItemSet iset )
		{
			return CMapperXmlTransformer.GetXmlString ( description, iset );
		}

		public static CMapperXmlItemSet FromXml ( string xml )
		{
			CMapperXmlItemSetCollection coll = CMapperXmlTransformer.ParseXmlString ( xml );
			if ( coll.Count > 0 )
				return coll [ 0 ];
			
			return null;
		}

		public string WriteToXml ( string description )
		{
			return CMapperXmlTransformer.GetXmlString ( description, this );
		}

		public void ReadFromXml ( string xml )
		{
			CMapperXmlItemSetCollection coll = CMapperXmlTransformer.ParseXmlString ( xml );
			if ( coll.Count > 0 )
			{
				CMapperXmlItemSet iset = coll [ 0 ];
			
				this.m_IdName = iset.IdName;
				this.m_Name = iset.Name;				
				this.m_SubSets = iset.SubSets;
				this.list = iset.list;
				//this.m_ParentCollection = iset.ParentCollection;
			}
		}
		#endregion
	
		#region Self Collection Operations
		public int Add ( CMapperXmlItem value )
		{
			if (value == null)
				throw new ArgumentNullException();
			return list.Add ( value );
		}

		public void Insert(int index, CMapperXmlItem value)
		{			
			if (value == null)
				throw new ArgumentNullException();
			list.Insert ( index, value );
		}

		public void Remove ( CMapperXmlItem value )
		{
			if (value == null)
				throw new ArgumentNullException();
			list.Remove ( value );
		}

		public void RemoveAt ( int index )
		{
			list.RemoveAt ( index );
		}
		public bool Contains(CMapperXmlItem value)
		{
			if (value == null)
				return false;
			return list.Contains(value);
		}
		public void Clear()
		{
			list.Clear();
		}
		public int IndexOf(CMapperXmlItem value)
		{			
			if (value == null)
				return -1;
			return list.IndexOf ( value );			
		}
		public CMapperXmlItem this[int index]
		{
			get
			{
				return (CMapperXmlItem) list[index];				
			}
			set
			{
				if (value == null)
					throw new ArgumentNullException();
				list[index] = value;
			}
		}

		#endregion

		#region ICollection Members

		public bool IsSynchronized
		{
			get
			{
				return list.IsSynchronized;
			}
		}

		public int Count
		{
			get
			{
				return list.Count;
			}
		}

		public void CopyTo(Array array, int index)
		{
			list.CopyTo(array, index);
		}

		public object SyncRoot
		{
			get
			{
				return list.SyncRoot;
			}
		}

		#endregion

		#region IEnumerable Members

		public IEnumerator GetEnumerator()
		{
			return list.GetEnumerator();
		}

		#endregion

		#region IList Members

		public bool IsReadOnly
		{
			get
			{				
				return false;
			}
		}

		object IList.this[int index]
		{
			get
			{
				return this[index];
			}
			set
			{
				if (value == null)
					throw new ArgumentNullException();
				if (!( value is CMapperXmlItem ))
					throw new ArgumentException();
				this[index] = (CMapperXmlItem)value;
			}
		}

		void IList.Insert(int index, object value)
		{
			if (value == null)
				throw new ArgumentNullException();
			if (!( value is CMapperXmlItem ))
				throw new ArgumentException();
			this.Insert ( index, (CMapperXmlItem)value );
		}

		void IList.Remove(object value)
		{
			if (value == null)
				throw new ArgumentNullException();
			if (!( value is CMapperXmlItem ))
				throw new ArgumentException();
			this.Remove((CMapperXmlItem)value);
		}

		bool IList.Contains(object value)
		{			
			if (value == null)
				return false;
			if (!( value is CMapperXmlItem ))
				throw new ArgumentException();
			return this.Contains((CMapperXmlItem)value);
		}
		
		int IList.IndexOf(object value)
		{	
			if (value == null)
				return -1;

			if (!( value is CMapperXmlItem ))
				throw new ArgumentException();
			return this.IndexOf((CMapperXmlItem)value);
		}

		int IList.Add(object value)
		{
			if (!( value is CMapperXmlItem ))
				throw new ArgumentException();
			return this.Add((CMapperXmlItem)value);			
		}

		public bool IsFixedSize
		{
			get
			{				
				return false;
			}
		}

		#endregion
	}
	#endregion

	#region CMapperXmlItemSetCollection
	[Serializable]
	public class CMapperXmlItemSetCollection : IList
	{
		#region Fields & Properties

		private ArrayList list;
		
		private string m_Name = "";
		/// <summary>
		///     
		///    name  
		/// </summary>
		public string Name
		{
			get { return m_Name; }
			set { m_Name = value; }
		}

		private CMapperXmlItemSet m_ParentSet = null;
		/// <summary>
		/// -   (ItemSet).
		/// </summary>
		public CMapperXmlItemSet ParentSet
		{
			get { return m_ParentSet; }
			set { m_ParentSet = value; }
		}

		#endregion

		#region Constructor

		public CMapperXmlItemSetCollection ( ):base ( )
		{
			list = new ArrayList ( );
		}

		public CMapperXmlItemSetCollection ( string name ):this ( )
		{
			m_Name = name;
		}

		#endregion

		#region Xml
		public static string ToXml ( string description, CMapperXmlItemSetCollection coll )
		{
			return CMapperXmlTransformer.GetXmlString ( description, coll );
		}

		public static void ToXmlFile ( string description, CMapperXmlItemSetCollection coll, string path )
		{
			CMapperXmlTransformer.CreateXmlFile ( description, coll, path );
		}

		public static CMapperXmlItemSetCollection FromXml ( string xml )
		{
			return CMapperXmlTransformer.ParseXmlString ( xml );			
		}
		
		public static CMapperXmlItemSetCollection FromXmlFile ( string path )
		{
			return CMapperXmlTransformer.ParseXmlFile ( path );			
		}

		public string WriteToXml ( string description )
		{
			return CMapperXmlTransformer.GetXmlString ( description, this );
		}

		public void ReadFromXml ( string xml )
		{
			CMapperXmlItemSetCollection coll = CMapperXmlTransformer.ParseXmlString ( xml );

			this.m_Name = coll.Name;
			this.list = coll.list;
			//this.m_ParentSet = coll.ParentSet;
		}
		#endregion

		#region Self
		
		public int Add ( CMapperXmlItemSet value )
		{
			if (value == null)
				throw new ArgumentNullException();
			return list.Add ( value );
		}

		public void Insert(int index, CMapperXmlItemSet value)
		{			
			if (value == null)
				throw new ArgumentNullException();
			list.Insert ( index, value );
		}

		public void Remove ( CMapperXmlItemSet value )
		{
			if (value == null)
				throw new ArgumentNullException();
			list.Remove ( value );
		}

		public void RemoveAt ( int index )
		{
			list.RemoveAt ( index );
		}
		public bool Contains(CMapperXmlItemSet value)
		{
			if (value == null)
				return false;
			return list.Contains(value);
		}
		public void Clear()
		{
			list.Clear();
		}
		public int IndexOf(CMapperXmlItemSet value)
		{			
			if (value == null)
				return -1;
			return list.IndexOf ( value );			
		}
		public CMapperXmlItemSet this[int index]
		{
			get
			{
				return (CMapperXmlItemSet) list[index];				
			}
			set
			{
				if (value == null)
					throw new ArgumentNullException();
				list[index] = value;
			}
		}

		#endregion			

		#region ICollection Members

		public bool IsSynchronized
		{
			get
			{
				return list.IsSynchronized;
			}
		}

		public int Count
		{
			get
			{
				return list.Count;
			}
		}

		public void CopyTo(Array array, int index)
		{
			list.CopyTo(array, index);
		}

		public object SyncRoot
		{
			get
			{
				return list.SyncRoot;
			}
		}

		#endregion

		#region IEnumerable Members

		public IEnumerator GetEnumerator()
		{
			return list.GetEnumerator();
		}

		#endregion

		#region IList Members

		public bool IsReadOnly
		{
			get
			{				
				return false;
			}
		}

		object IList.this[int index]
		{
			get
			{
				return this[index];
			}
			set
			{
				if (value == null)
					throw new ArgumentNullException();
				if (!( value is CMapperXmlItemSet ))
					throw new ArgumentException();
				this[index] = (CMapperXmlItemSet)value;
			}
		}

		void IList.Insert(int index, object value)
		{
			if (value == null)
				throw new ArgumentNullException();
			if (!( value is CMapperXmlItemSet ))
				throw new ArgumentException();
			this.Insert ( index, (CMapperXmlItemSet)value );
		}

		void IList.Remove(object value)
		{
			if (value == null)
				throw new ArgumentNullException();
			if (!( value is CMapperXmlItemSet ))
				throw new ArgumentException();
			this.Remove((CMapperXmlItemSet)value);
		}

		bool IList.Contains(object value)
		{			
			if (value == null)
				return false;
			if (!( value is CMapperXmlItemSet ))
				throw new ArgumentException();
			return this.Contains((CMapperXmlItemSet)value);
		}
		
		int IList.IndexOf(object value)
		{	
			if (value == null)
				return -1;

			if (!( value is CMapperXmlItemSet ))
				throw new ArgumentException();
			return this.IndexOf((CMapperXmlItemSet)value);
		}

		int IList.Add(object value)
		{
			if (!( value is CMapperXmlItemSet ))
				throw new ArgumentException();
			return this.Add((CMapperXmlItemSet)value);			
		}

		public bool IsFixedSize
		{
			get
			{				
				return false;
			}
		}

		#endregion
	}
	#endregion
}
